@@ -4,7 +4,6 @@
 #include <iostream>
 #include <stdlib.h>
 #include <assert.h>
-#include <qregexp.h>
 
 #include <QDebug>
 #include <QFile>
@@ -30,7 +29,7 @@ static const char *singletags[] = {"beginpage","imagedata", "colspec", "spanspec
                                    "anchor", "xref", "area",
                                    "footnoteref", "void", "inlinegraphic",
                                    "glosssee", "graphic", "xi:include",
-                                   0};
+                                   nullptr};
 static const char *cuttingtags[] = {"bridgehead", "trans_comment", "para", "title", "term",
                                     "entry", "contrib", "keyword", "example",
                                     "note", "footnote", "caution",
@@ -63,15 +62,15 @@ static const char *cuttingtags[] = {"bridgehead", "trans_comment", "para", "titl
                                     "revdescription", "glossentry", "partinfo",
 				    "segmentedlist", "segtitle", "seg", "seglistitem", "screenco",
 				    "titleabbrev", "date", "authorinitials", "holder", "releaseinfo",
-                                    0};
+                                    nullptr};
 static const char *literaltags[] = {"literallayout", "synopsis", "screen",
-				    "programlisting", 0};
+				    "programlisting", nullptr};
 
 void StructureParser::startDocument()
 {
-    infos_reg = QRegExp("\\s*poxml_line=\"(\\d+)\" poxml_col=\"(\\d+)\"");
-    do_not_split_reg = QRegExp("\\s*condition=\"do-not-split\"");
-    message = "";
+    infos_reg = QRegularExpression(QStringLiteral("\\s*poxml_line=\"(\\d+)\" poxml_col=\"(\\d+)\""));
+    do_not_split_reg = QRegularExpression(QStringLiteral("\\s*condition=\"do-not-split\""));
+    message = QString();
     inside = 0;
 }
 
@@ -86,7 +85,7 @@ bool StructureParser::isCuttingTag(const QString &qName)
     return isLiteralTag(qName);
 }
 
-bool StructureParser::isSingleTag(const QStringRef &qName)
+bool StructureParser::isSingleTag(QStringView qName)
 {
     int index = 0;
     while (singletags[index]) {
@@ -108,13 +107,13 @@ bool StructureParser::isLiteralTag(const QString &qName)
     return false;
 }
 
-void StructureParser::skippedEntity ( const QStringRef & name )
+void StructureParser::skippedEntity ( QStringView name )
 {
     if (inside)
         message += QString::fromLatin1("&%1;").arg(name);
 }
 
-void StructureParser::startElement( int lineNumber, int columnNumber, const QStringRef& qName, const QXmlStreamAttributes & attr )
+void StructureParser::startElement( int lineNumber, int columnNumber, QStringView qName, const QXmlStreamAttributes & attr )
 {
     const QString tname = qName.toString().toLower();
 
@@ -133,7 +132,7 @@ void StructureParser::startElement( int lineNumber, int columnNumber, const QStr
 
     if (inside)
     {
-        QString tmp = '<' + tname;
+        QString tmp = QLatin1Char('<') + tname;
         for (int i = 0; i < attr.length(); i++) {
             tmp += QString::fromLatin1(" %1=\"%2\"").arg(attr[i].qualifiedName(), attr[i].value());
         }
@@ -143,19 +142,19 @@ void StructureParser::startElement( int lineNumber, int columnNumber, const QStr
         if (isSingleTag(qName))
             tmp += QString::fromLatin1("/>");
         else
-            tmp += '>';
+            tmp += QLatin1Char('>');
         message += tmp;
         if (first)
             startcol -= message.length();
     }
 
-    if (tname == QLatin1String("anchor") || tname.leftRef(4) == QLatin1String("sect") || tname == QLatin1String("chapter"))
+    if (tname == QLatin1String("anchor") || QStringView(tname).left(4) == QLatin1String("sect") || tname == QLatin1String("chapter"))
         if (!attr.value("id").isEmpty()) list.pc.addAnchor(attr.value("id").toString());
 }
 
 bool StructureParser::isClosure(const QString &message)
 {
-    assert(message.at(0) == '<');
+    assert(message.at(0) == QLatin1Char('<'));
     int endindex = 1;
     while (!message.at(endindex).isSpace() && message.at(endindex) != QLatin1Char('>'))
         endindex++;
@@ -174,8 +173,8 @@ bool StructureParser::closureTag(const QString& message, const QString &tag)
     const int messageLength = message.length();
     while (true)
     {
-        int nextclose = message.indexOf(QRegExp(QString::fromLatin1("</%1[\\s>]").arg(tag)), index);
-        int nextstart = message.indexOf(QRegExp(QString::fromLatin1("<%1[>\\s]").arg(tag)), index);
+        int nextclose = message.indexOf(QRegularExpression(QString::fromLatin1("</%1[\\s>]").arg(tag)), index);
+        int nextstart = message.indexOf(QRegularExpression(QString::fromLatin1("<%1[>\\s]").arg(tag)), index);
         //  qDebug("finding %d %d %d %d", nextstart, nextclose, index, inside);
         if (nextclose == -1) {
 #ifdef POXML_DEBUG
@@ -220,19 +219,19 @@ void StructureParser::descape(QString &message)
             case '\t':
             case '\r':
                 if (!inside)
-                    message[index] = ' ';
+                    message[index] = QLatin1Char(' ');
             case ' ':
                 if (!inside && lastws)
-                    message[index] = '\010';
+                    message[index] = QLatin1Char('\010');
                 lastws = true;
                 break;
             case '<': {
                 int endindex = index+1;
                 while (endindex < message.length() && !message.at(endindex).isSpace() &&
-                       message.at(endindex) != '>')
+                       message.at(endindex) != QLatin1Char('>'))
                     endindex++;
                 QString tag = message.mid(index + 1, endindex - index - 1);
-                if (tag.at(0) == '/') {
+                if (tag.at(0) == QLatin1Char('/')) {
                     if (isLiteralTag(tag.mid(1)))
                         inside--;
                 } else
@@ -246,7 +245,7 @@ void StructureParser::descape(QString &message)
 
         index++;
     }
-    message.remove('\010');
+    message.remove(QLatin1Char('\010'));
 }
 
 bool StructureParser::formatMessage(MsgBlock &msg) const
@@ -272,7 +271,7 @@ bool StructureParser::formatMessage(MsgBlock &msg) const
         int slen = strlen(singletags[index]);
 
         if (!msg.msgid.isEmpty() &&
-            msg.msgid.at(0) == QLatin1Char('<') && msg.msgid.midRef(1, slen) == QLatin1String(singletags[index]) &&
+            msg.msgid.at(0) == QLatin1Char('<') && QStringView(msg.msgid).mid(1, slen) == QLatin1String(singletags[index]) &&
             !msg.msgid.at( slen + 1 ).isLetterOrNumber() )
         {
 #ifdef POXML_DEBUG
@@ -289,7 +288,7 @@ bool StructureParser::formatMessage(MsgBlock &msg) const
         }
     }
 
-    while (msg.msgid.right(2) == "/>" && msg.msgid.startsWith(QLatin1Char('<')))
+    while (msg.msgid.right(2) == QStringLiteral("/>") && msg.msgid.startsWith(QLatin1Char('<')))
     {
         int strindex = msg.msgid.length() - 2;
         strindex = msg.msgid.lastIndexOf(QLatin1Char('<'), strindex);
@@ -312,7 +311,7 @@ bool StructureParser::formatMessage(MsgBlock &msg) const
             strindex++;
         QString starttag = msg.msgid.mid(1, strindex - 1);
         int endindex = msg.msgid.length() - 2;
-        while (msg.msgid.at(endindex) != QLatin1Char('<') && msg.msgid.at(endindex + 1) != '/')
+        while (msg.msgid.at(endindex) != QLatin1Char('<') && msg.msgid.at(endindex + 1) != QLatin1Char('/'))
             endindex--;
 #ifdef POXML_DEBUG
         qDebug("endIndex %d", endindex);
@@ -332,7 +331,7 @@ bool StructureParser::formatMessage(MsgBlock &msg) const
             strindex = msg.msgid.indexOf(QLatin1Char('>'), 0);
             QString attr = msg.msgid.left(strindex);
             QString real_attr = attr.mid(starttag.length() + 1);
-            const int real_attr_infos_reg_pos = infos_reg.indexIn(real_attr);
+            const int real_attr_infos_reg_pos = infos_reg.match(real_attr).capturedStart();
             if (real_attr_infos_reg_pos >= 0) {
                 real_attr = real_attr.left(real_attr_infos_reg_pos);
             }
@@ -344,20 +343,20 @@ bool StructureParser::formatMessage(MsgBlock &msg) const
             stripWhiteSpace( msg.msgid );
             msg.tag = starttag;
 
-            if (infos_reg.indexIn(attr) >= 0) {
-                msg.lines.first().start_line = infos_reg.cap(1).toInt();
-                msg.lines.first().start_col = infos_reg.cap(2).toInt();
+            if (const auto match = infos_reg.match(attr); match.hasMatch()) {
+                msg.lines.first().start_line = match.capturedView(1).toInt();
+                msg.lines.first().start_col = match.capturedView(2).toInt();
 #ifdef POXML_DEBUG
                 qDebug("col %s %s %d", qPrintable(attr), qPrintable(msg.msgid), msg.lines.first().start_col);
 #endif
                 offset = 0;
 
-                if (infos_reg.indexIn(endtag_attr) >= 0) {
-                    msg.lines.first().end_line = infos_reg.cap(1).toInt();
-                    msg.lines.first().end_col = infos_reg.cap(2).toInt() + 1;
+                if (const auto match = infos_reg.match(endtag_attr); match.hasMatch()) {
+                    msg.lines.first().end_line = match.capturedView(1).toInt();
+                    msg.lines.first().end_col = match.capturedView(2).toInt() + 1;
                 }
             }
-            if (do_not_split_reg.indexIn(attr) >= 0) {
+            if (do_not_split_reg.match(attr).hasMatch()) {
                 msg.do_not_split = true;
                 break;
             }
@@ -418,9 +417,9 @@ MsgList StructureParser::splitMessage(const MsgBlock &mb)
 #endif
 
                 // the exception for poxml_* attributes is made in the closing tag
-                int closing_index = message.indexOf(QRegExp(QString::fromLatin1("</%1[\\s>]").arg(tag)),
+                int closing_index = message.indexOf(QRegularExpression(QString::fromLatin1("</%1[\\s>]").arg(tag)),
                                                  strindex);
-                int starting_index = message.indexOf(QRegExp(QString::fromLatin1("<%1[\\s>]").arg(tag)),
+                int starting_index = message.indexOf(QRegularExpression(QString::fromLatin1("<%1[\\s>]").arg(tag)),
                                                   strindex);
 
 #ifdef POXML_DEBUG
@@ -529,9 +528,9 @@ MsgList StructureParser::splitMessage(const MsgBlock &mb)
                 qDebug("inside %s %d", qPrintable(message.mid(strindex, 35)), inside);
 #endif
 
-                int closing_index = message.lastIndexOf(QRegExp(QString::fromLatin1("</%1[\\s>]").arg(tag)),
+                int closing_index = message.lastIndexOf(QRegularExpression(QString::fromLatin1("</%1[\\s>]").arg(tag)),
                                                     strindex - 1);
-                int starting_index = message.lastIndexOf(QRegExp(QString::fromLatin1("<%1[\\s>]").arg(tag)),
+                int starting_index = message.lastIndexOf(QRegularExpression(QString::fromLatin1("<%1[\\s>]").arg(tag)),
                                                      strindex - 1);
 
 #ifdef POXML_DEBUG
@@ -593,7 +592,7 @@ error:
     return result;
 }
 
-void StructureParser::endElement( int lineNumber, int columnNumber, const QStringRef& qName)
+void StructureParser::endElement( int lineNumber, int columnNumber, QStringView qName)
 {
     const QString tname = qName.toString().toLower();
 
@@ -632,9 +631,9 @@ void StructureParser::endElement( int lineNumber, int columnNumber, const QStrin
                 // if the remaining text still starts with a tag, the poxml_ info
                 // is most probably more correct
                 if (!(*it).msgid.isEmpty() && (*it).msgid.at(0) == QLatin1Char('<') && isClosure((*it).msgid)) {
-                    if (infos_reg.indexIn((*it).msgid) >= 0) {
-                        (*it).lines.first().start_line = infos_reg.cap(1).toInt();
-                        (*it).lines.first().start_col =  infos_reg.cap(2).toInt();;
+                    if (const auto match = infos_reg.match((*it).msgid); match.hasMatch()) {
+                        (*it).lines.first().start_line = match.capturedView(1).toInt();
+                        (*it).lines.first().start_col = match.capturedView(2).toInt();;
                         (*it).lines.first().offset = 0;
                     }
                 }
@@ -647,7 +646,7 @@ void StructureParser::endElement( int lineNumber, int columnNumber, const QStrin
     }
 }
 
-void StructureParser::comment ( const QStringRef &c )
+void StructureParser::comment ( QStringView c )
 {
     if (c.left(7) != QLatin1String(" TRANS:"))
         return;
@@ -708,8 +707,8 @@ void StructureParser::cleanupTags( QString &contents )
     contents.replace(QChar::fromLatin1('&'), QString::fromLatin1("!POXML_AMP!"));
 
     for (int index = 0; literaltags[index]; index++) {
-        QRegExp start(QString::fromLatin1("<%1[\\s>]").arg(literaltags[index]));
-        QRegExp end(QString::fromLatin1("</%1[\\s>]").arg(literaltags[index]));
+        const QRegularExpression start(QString::fromLatin1("<%1[\\s>]").arg(QString::fromLatin1(literaltags[index])));
+        const QRegularExpression end(QString::fromLatin1("</%1[\\s>]").arg(QString::fromLatin1(literaltags[index])));
         int strindex = 0;
         while (true) {
             strindex = contents.indexOf(start, strindex);
@@ -726,50 +725,53 @@ void StructureParser::cleanupTags( QString &contents )
         }
     }
 
-    QRegExp unclosed("</(\\w*)\\s\\s*>");
-    int index = -1;
+    const QRegularExpression unclosed(QStringLiteral("</(\\w*)\\s\\s*>"));
+    int index = 0;
     while (true) {
-        index = unclosed.indexIn(contents, index + 1);
-        if (index < 0)
+        const auto match = unclosed.match(contents, index );
+        if (!match.hasMatch())
             break;
-        QString tag = unclosed.cap(1);
-        contents.replace(index, unclosed.matchedLength(), QString::fromLatin1("</%1>").arg(tag));
+        index = match.capturedStart();
+        QString tag = match.captured(1);
+        contents.replace(index, match.capturedLength(), QString::fromLatin1("</%1>").arg(tag));
     }
 
-    QRegExp start("<((\\s*[^<>\\s])*)\\s\\s*(/*)>");
-    start.setMinimal(true);
+    const QRegularExpression start(QStringLiteral("<((\\s*[^<>\\s])*)\\s\\s*(/*)>"), QRegularExpression::InvertedGreedinessOption);
 
     index = -1;
     while (true) {
-        index = start.indexIn(contents, index + 1);
-        if (index < 0)
+        const auto match = start.match(contents, index + 1);
+        if (!match.hasMatch())
             break;
-        QString tag = start.cap(1);
-	QString cut = start.capturedTexts().last();
+        index = match.capturedStart();
+        QString tag = match.captured(1);
+	QString cut = match.capturedTexts().last();
         // qDebug("UNCLO %s %d -%s- -%s-", qPrintable(start.cap(0)), index, qPrintable(tag), qPrintable(cut));
-        contents.replace(index, start.matchedLength(), QString::fromLatin1("<%1%2>").arg(tag, cut));
+        contents.replace(index, match.capturedLength(), QString::fromLatin1("<%1%2>").arg(tag, cut));
     }
-    QRegExp singletag("<(\\w*)\\s([^><]*)/>");
+    const QRegularExpression singletag(QStringLiteral("<(\\w*)\\s([^><]*)/>"));
 
     index = -1;
     while (true) {
-        index = singletag.indexIn(contents, index + 1);
-        if (index < 0)
+        const auto match = singletag.match(contents, index + 1);
+        if (!match.hasMatch())
             break;
-        const QString tag = singletag.cap(1);
-        if (!StructureParser::isSingleTag(QStringRef(&tag))) {
-            contents.replace(index, singletag.matchedLength(), QString::fromLatin1("<%1 %2></%3>").arg(tag, singletag.cap(2), tag));
+        index = match.capturedStart();
+        const QString tag = match.captured(1);
+        if (!StructureParser::isSingleTag(tag)) {
+            contents.replace(index, match.capturedLength(), QString::fromLatin1("<%1 %2></%3>").arg(tag, match.captured(2), tag));
         }
     }
 
-    QRegExp trans_comment("<!-- TRANS:([^<>]*)-->");
+    const QRegularExpression trans_comment(QStringLiteral("<!-- TRANS:([^<>]*)-->"));
     index = -1;
     while (true) {
-        index = trans_comment.indexIn(contents, index + 1);
-        if (index < 0)
+        const auto match = trans_comment.match(contents, index + 1);
+        if (!match.hasMatch())
             break;
-        QString msgid = trans_comment.cap(1);
-        contents.replace(index, trans_comment.matchedLength(), QString::fromLatin1("<trans_comment>%1</trans_comment>").arg(msgid));
+        index = match.capturedStart();
+        QString msgid = match.captured(1);
+        contents.replace(index, match.capturedLength(), QString::fromLatin1("<trans_comment>%1</trans_comment>").arg(msgid));
     }
 
 #ifdef POXML_DEBUG
@@ -782,14 +784,15 @@ static bool removeEmptyTag( QString &contents, const QString & tag)
 {
 //    qDebug("cont %s %s", qPrintable(contents), qPrintable(tag));
 
-    QRegExp empty(QString::fromLatin1("<%1[^>]*>[\\s\n]+</%2\\s*>").arg(tag, tag));
+    const QRegularExpression empty(QString::fromLatin1("<%1[^>]*>[\\s\n]+</%2\\s*>").arg(tag, tag));
     int strindex = 0;
     while (true) {
-        strindex = contents.indexOf(empty, strindex);
-        if (strindex < 0)
+        const auto match = empty.match(contents, strindex);
+        if (!match.hasMatch())
             break;
+        strindex = match.capturedStart();
         qDebug("found empty tag %s", qPrintable(tag));
-        contents.replace(strindex, empty.matchedLength(), ' ');
+        contents.replace(strindex, match.capturedLength(), QLatin1Char(' '));
         strindex++;
         return true;
     }
@@ -816,7 +819,7 @@ void StructureParser::removeEmptyTags( QString &contents )
     } while (removed);
 }
 
-void StructureParser::characters(const QStringRef &ch)
+void StructureParser::characters(QStringView ch)
 {
     if (inside && !ch.isEmpty())
         message += ch;
@@ -858,7 +861,7 @@ static MsgList parseContents(const QString &contents)
                 // QXmlStreamReader is annoying and won't accept custom entities like &POXML_LINEFEED;
                 // unless the file has a DTD, so if we fail parsing, inject a DTD into the XML contents and parse again
                 if (!foundDtd) {
-                     const QRegularExpression xmlDeclarationRegExp("^.*<\\?xml.*?\\?>");
+                     const QRegularExpression xmlDeclarationRegExp(QStringLiteral("^.*<\\?xml.*?\\?>"));
                      const QRegularExpressionMatch match = xmlDeclarationRegExp.match(contents);
                      const QString fakeDTD = QStringLiteral("<!DOCTYPE fake PUBLIC \"fake\" \"fake\" []>");
                      if (match.hasMatch()) {
@@ -896,23 +899,28 @@ MsgList parseXML(const char *filename)
     {
         // find internal entities that start with "i18n-", and extract
         // their replacement texts:
-        QRegExp rx( "<!ENTITY\\s+([^\\s]+)\\s+([\"'])" );
-        for ( int index = rx.indexIn( contents, 0 ) ; index >= 0 ; index = rx.indexIn( contents, index ) ) {
-            const QString name = rx.cap( 1 );
-            const QChar delim = rx.cap( 2 ).at( 0 );
+        const QRegularExpression rx( QStringLiteral("<!ENTITY\\s+([^\\s]+)\\s+([\"'])") );
+        for ( int index = 0 ;; ) {
+            const auto match = rx.match(contents, index);
+            if (!match.hasMatch()) {
+                break;
+            }
+            index = match.capturedStart();
+            const QString name = match.captured( 1 );
+            const QChar delim = match.captured( 2 ).at( 0 );
             const int start = index;
-            index = contents.indexOf( delim, index + rx.matchedLength() );
-            index = contents.indexOf( '>', index );
+            index = contents.indexOf( delim, index + match.capturedLength() );
+            index = contents.indexOf( QLatin1Char('>'), index );
             if ( !name.startsWith( QLatin1String("i18n-") ) )
                 continue;
             const QString entity = contents.mid( start, index - start + 1 );
             MsgBlock block;
-            block.tag = "!ENTITY";
+            block.tag = QStringLiteral("!ENTITY");
             BlockInfo bi;
-            bi.start_line = countRev( contents, '\n', index ) + 1;
-            bi.start_col  = start - contents.lastIndexOf( '\n', start ) - 1;
-            bi.end_line   = bi.start_line + entity.count( '\n' );
-            bi.end_col    = index - contents.lastIndexOf( '\n', index ) + 1;
+            bi.start_line = countRev( contents, QLatin1Char('\n'), index ) + 1;
+            bi.start_col  = start - contents.lastIndexOf( QLatin1Char('\n'), start ) - 1;
+            bi.end_line   = bi.start_line + entity.count( QLatin1Char('\n') );
+            bi.end_col    = index - contents.lastIndexOf( QLatin1Char('\n'), index ) + 1;
 #ifdef POXML_DEBUG
             qDebug( "ENTITY %s @ i:%d l:%d c:%d->l:%d c:%d", qPrintable( name ),
                     index, bi.start_line, bi.start_col, bi.end_line, bi.end_col );
@@ -925,13 +933,13 @@ MsgList parseXML(const char *filename)
 
     // Remove all entity definitions now:
     while (true) {
-        int index = contents.indexOf("<!ENTITY");
+        int index = contents.indexOf(QStringLiteral("<!ENTITY"));
         if (index < 0)
             break;
         int inside = 0;
         int endindex = index + 1;
-        QString replacement = "";
-        while (contents.at(endindex) != '>' || inside)
+        QString replacement;
+        while (contents.at(endindex) != QLatin1Char('>') || inside)
         {
             switch (contents.at(endindex).toLatin1()) {
                 case '<':
@@ -939,7 +947,7 @@ MsgList parseXML(const char *filename)
                 case '>':
                     inside--; break;
                 case '\n':
-                    replacement += '\n';
+                    replacement += QLatin1Char('\n');
                     break;
                 default:
                     break;
